package PureMessage::Monitor::Config;

use strict;
use PureMessage::Config qw(expand_config_filename);

use base qw( PureMessage::Config::Validator Exporter );

# allow for our configuration to be exported
our @EXPORT_OK = qw( %config );
our %config;

# refresh our configuration
refresh();
sub refresh {
    require PureMessage::Config::Validator;

    my $parser = ActiveState::Config->new( strict=>1, allow_includes=>1 );
    PureMessage::Config::Validator->define_core_validators( $parser );
    PureMessage::Monitor::Config->validate( $parser );

    my $cfgfile = expand_config_filename( 'monitor' );
    $parser->parse_file( $cfgfile );

    %config = $parser->as_hash();
}

# validate our configuration
sub validate {
    my ($pkg, $p) = @_;

    # Define the validation rules and their default values
    my %conf = (
        # all notifiers need to be implemented by a module, each of which
        # defines its own config for matching things to generate events
        'notifier#*.use'        => [ '#str' ],
        'notifier#*.event#*.*'  => [ '#any' ],
        # all analyzers need to be implemented by a module, each of which
        # can add to this config with custom configuration directives
        'analyzer#*.use'        => [ '#str' ],
        'analyzer#*.event'      => [ '#str' ],
        'analyzer#*.expiry'     => [ '#duration' ],
        'analyzer#*.period'     => [ '#duration' ],
            # analyzer: threshold
        'analyzer#*.threshold'  => [ '#int' ],  # resource init is strict
        'analyzer#*.field'      => [ '#str' ],  # resource init is strict
        # all reactors need to be implemented by a module, each of which
        # can add to this config with custom configuration directives
        'reactor#*.use'         => [ '#str' ],
        'reactor#*.event'       => [ '#str' ],
            # reactor: email
        'reactor#*.to'          => [ '#str' ],  # resource init is strict
        'reactor#*.from'        => [ '#str' ],  # resource init is strict
        'reactor#*.subject'     => [ '#str' ],  # resource init is strict
        'reactor#*.template'    => [ '#str' ],  # resource init is strict
            # reactor: log
        'reactor#*.logfile'     => [ '#str' ],  # resource init is strict
        'reactor#*.format'      => [ '#str' ],  # resource init is strict
            # reactor: add to list
        'reactor#*.list'        => [ '#str' ],  # resource init is strict
            # reactor: add to list temporarily
        'reactor#*.expiry'      => [ '#str' ],  # resource init is strict
    );

    # Set the rules/defaults into the parser
    foreach my $key (keys %conf) {
        my ($type, $def) = @{$conf{$key}};
        $p->typedecl( $key, $type ) if ($type);
        $p->default( $key, $def )   if ($def);
    }
}

1;
